;;; irc.el
;;;
;;; copyright (C) 1996 lars brinkhoff
;;;
;;; written 961004-961011
;;; heavy modifications 961129-961204
;;; 961208 - wow  really beeps now :)
;;; 961229 - imp4
;;; 970130 - ctcp input

;;; 1 inch == 2.54 cm
;;; 1 pound == 0.45359 kg == 16 ounces
;;; x C == (9y/5 + 32) F, y F == 5(x - 32)/9 C
;;; 1 cup == 0.23659 litres == 16 tablespoons == 48 teaspoons == 0.5 us pint


(defvar irc-filter-leftover ""
  "Input not yet processed.")

(defvar irc-nick nil
  "Current user nickname.")

(defvar irc-stream nil
  "Connection to the IRC server.")

(defvar irc-stream-nick-alist nil
  "Nickname used on a server connection.")

(defvar irc-stream-alist nil
  "Associates streams (server connections) with channels and their buffers.")

(defvar irc-mode-line-channel nil
  "Channel displayed in the mode line.")

(defun irc-filter (process input)
  "Receives everything from the IRC server."
  (let ((i 0) j)
    (setq input (concat irc-filter-leftover input))
    (while (setq j (string-match "\r\n" input i))
      (irc-receive (substring input i j) process)
      (setq i (+ j 2)))
    (setq irc-filter-leftover (substring input i))))

(defun irc-input (string)
  "Receives all user input."
  (interactive "s")
  (cond
   ((string-match "^/\\([a-zA-Z]*\\) \\(.*\\)$" string)
    (irc-command (match-string 1 string) (match-string 2 string)))
   ((string-match "^/\\([a-zA-Z]*\\)$" string)
    (irc-command (match-string 1 string)))
   (t
    (irc-command nil string))))

(defun irc-start-input ()
  "Called when the user presses a self-inserting key in an IRC buffer."
  (interactive)
  (let ((string (char-to-string (aref (recent-keys)
				      (1- (length (recent-keys)))))))
    (irc-read-input string)))

(defun irc-read-input (&optional string)
  "Reads user input in the minibuffer."
  (interactive)
  (if (not string)
      (setq string ""))
  (irc-input (read-from-minibuffer "> " string)))

(defun irc-mode ()
  "Description that will appear when C-h m is typed.
\[COMMAND], \{KEYMAP}, \<KEYMAP>"
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'irc-mode)
  (setq mode-name "IRC")
  (use-local-map irc-mode-map)
  ;; syntax table, irc-mode-syntax-table
  ;; irc-mode-abbrev-table
  ;; local variables, (make-local-variable ...)
  (put 'irc-mode 'mode-class 'special)
  (setq irc-mode-line-channel nil)
  (make-local-variable 'global-mode-string)
  (display-time)
  (setq mode-line-format '("" global-mode-string " " irc-nick " "
			   irc-mode-line-channel " (+modes)"))
  (run-hooks 'irc-mode-hook))

(defvar irc-mode-map nil
  "Local keymap for IRC buffers.")

(defun irc ()
  "Starts a new IRC session."
  (interactive)
  (make-variable-buffer-local 'irc-nick)
  (make-variable-buffer-local 'irc-stream)
  (make-variable-buffer-local 'irc-mode-line-channel)
  (irc-open-window (irc-open-server
		    (nth 2
			 '("193.141.101.39" ; atlantis
			   "205.150.207.3" ; insane, or whatever
			   "130.241.30.143" ; new hades, irc.nocrew
			   "130.241.152.48" ; hades
			   "129.16.117.11"
			   "127.0.0.1")) ; localhost
		    (or 7000 6667)
		    (user-login-name))))

(defun irc-open-server (host port nick)
  (or (stringp host) (signal 'wrong-type-argument host))
  (or (integerp port) (signal 'wrong-type-argument port))
  (let* ((buffer (generate-new-buffer "*irc*"))
	 (stream 
	  (condition-case err
	      (open-network-stream "irc" buffer host port)
	    (error
	     ;(irc-printf "*** Unable to connect to port %d of server %s: %s"
		;	 port host (cdr err))
	     (message "Unable to cononect to port %d of server %s: %s"
		      port host (cdr err))
	     nil))))
    (if stream
	(progn
	  (add-to-list 'irc-stream-alist (list stream))
	  (setq irc-stream stream)
	  (set-process-filter stream 'irc-filter)
	  (irc-sendf "USER %s 1 2 :http://www.nocrew.pp.se/lars" nick)
	  (irc-set-stream-nick stream nick)
	  (irc-command "nick" nick)
	  (list stream nick))
      nil)))

(defun irc-open-window (list)
  (let ((buffer (generate-new-buffer "*irc*"))
	(stream (car list))
	(nick (nth 1 list)))
    (split-window-vertically)
    (switch-to-buffer-other-window buffer)
    (irc-mode)
    (setq irc-stream stream
	  irc-nick nick)
    (irc-set-buffer-channel stream buffer nil)))

(if (not irc-mode-map)
    (progn
      (setq irc-mode-map (make-keymap))
      (substitute-key-definition 'self-insert-command 'irc-start-input
				 irc-mode-map global-map)
      (substitute-key-definition 'newline 'irc-read-input
				 irc-mode-map global-map)))

(let ((old-load-path load-path))
  (setq load-path (append load-path '(".")))
  (unwind-protect
      (progn
	(load "irc-util")
	(load "irc-receive")
	(load "irc-send")
	(load "irc-print")
	(load "irc-command"))
    (setq load-path old-load-path)))

(provide 'irc)

;;; end of irc.el
;; tjo
;; test
